from django.core.management.base import BaseCommand
from daily_digest.utils import send_daily_digest


class Command(BaseCommand):
    help = 'Send daily digest email.'

    def success_message(self, message):
        if hasattr(self.style, 'SUCCESS'):
            self.stdout.write(self.style.SUCCESS(message))
        else:
            # Django 1.8
            self.stdout.write(self.style.MIGRATE_SUCCESS(message))

    def handle(self, *args, **options):
        send_daily_digest()

        self.success_message('Sent daily digest')



# -*- coding: utf-8 -*-
from datetime import datetime, timedelta

import leather
import pytz
from cairosvg import svg2png
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.mail import EmailMultiAlternatives
from django.template.loader import get_template
from email.mime.image import MIMEImage
from premailer import transform

from .config import daily_digest_config

leather.theme.background_color = '#ffffff'
leather.theme.title_font_family = 'Helvetica'
leather.theme.legend_font_family = 'Helvetica'
leather.theme.tick_font_family = 'Helvetica'
PRIMARY_STROKE_COLOR = '#4383CC'
PREV_PERIOD_STROKE_COLOR = '#B4CDEB'

EMAIL_TIME_PERIOD = 7


def current_time_naive():
    return datetime.now()


def series_labels(series_1_count, series_2_count):
    if series_2_count > 0:
        percent_change = int(round((float(series_1_count - series_2_count) / series_2_count) * 100, 0))
        if percent_change > 0:
            percent_change = '+{}%'.format(percent_change)
        else:
            percent_change = '{}%'.format(percent_change)
    else:
        percent_change = ''

    current_label = 'Last {} Days ({}) {}'.format(EMAIL_TIME_PERIOD, series_1_count, percent_change)
    prev_period_label = 'Previous Period ({})'.format(series_2_count)
    return current_label, prev_period_label


def series_data_for_model(queryset, field, timezone, prev_period=False, exclude_today=False):
    start_of_today = timezone.localize(current_time_naive().replace(hour=0, minute=0, second=0))
    start_of_today = start_of_today.astimezone(pytz.UTC)

    if exclude_today:
        start_of_today = start_of_today - timedelta(days=1)

    start_of_today += timedelta(days=1)  # End of today
    if prev_period:
        period_end = start_of_today - timedelta(days=EMAIL_TIME_PERIOD)  # End of today
        period_start = period_end - timedelta(days=EMAIL_TIME_PERIOD)
    else:
        period_end = start_of_today
        period_start = period_end - timedelta(days=EMAIL_TIME_PERIOD)

    filters = {
        '{}__gte'.format(field): period_start,
        '{}__lt'.format(field): period_end
    }
    series_data = queryset.filter(**filters).values_list(
        field, flat=True
    )
    series_data = list(map(
        lambda x: x.astimezone(timezone).date(),
        series_data
    ))

    grouped_by_date = []
    period_start_local = period_start.astimezone(timezone)
    for i in range(0, EMAIL_TIME_PERIOD):
        day = (period_start_local + timedelta(days=i)).date()
        count = 0
        for user_date in series_data:
            if user_date == day:
                count += 1
        if prev_period:
            day = day + timedelta(days=EMAIL_TIME_PERIOD)  # offset date to show both series on one chart
        grouped_by_date.append([day, count])

    return grouped_by_date, len(series_data)


def svg_data_for_query(queryset, field, chart_name, timezone, exclude_today=False):
    data, total_count = series_data_for_model(
        queryset, field, timezone, exclude_today=exclude_today)
    data_prev_period, total_count_prev_period = series_data_for_model(
        queryset, field, timezone, prev_period=True, exclude_today=exclude_today)

    # Show every date on the x axis
    x_axis_ticks = []
    for item in data:
        x_axis_ticks.append(item[0])

    chart = leather.Chart()
    chart.add_x_axis(ticks=x_axis_ticks)

    # Start at 0 - Turn this into an option
    y_max = max([item[1] for item in data])
    y_max_prev_period = max([item[1] for item in data_prev_period])
    chart.add_y_scale(0, max([y_max, y_max_prev_period]))

    current_label, prev_period_label = series_labels(total_count, total_count_prev_period)
    chart.add_line(data, name=current_label, stroke_color=PRIMARY_STROKE_COLOR)
    chart.add_line(data_prev_period, name=prev_period_label, stroke_color=PREV_PERIOD_STROKE_COLOR, stroke_dasharray='5')
    chart.to_svg('/tmp/{}.svg'.format(chart_name), width=480, height=240)

    with open('/tmp/{}.svg'.format(chart_name)) as svgfile:
        svg_data = svgfile.read()
    svg2png(url='/tmp/{}.svg'.format(chart_name), write_to='/tmp/{}.png'.format(chart_name), scale=2)

    return svg_data


def charts_data_for_config(chart_format='svg'):
    charts_data = []
    for chart_config in daily_digest_config.chart_configs:
        title = chart_config.title
        app_label = chart_config.app_label
        model = chart_config.model
        date_field = chart_config.date_field
        content_type = ContentType.objects.get(app_label=app_label, model=model)
        filter_kwargs = chart_config.filter_kwargs
        queryset = content_type.model_class().objects.filter(**filter_kwargs)
        if chart_config.distinct_by:
            queryset = queryset.distinct(
                chart_config.distinct_by
            ).order_by(
                chart_config.distinct_by
            )
        result = {
            'title': title,
            'svg_data': svg_data_for_query(
                queryset, date_field, chart_config.slug,
                timezone=daily_digest_config.timezone,
                exclude_today=daily_digest_config.exclude_today
            )
        }
        if chart_format == 'png':
            del result['svg_data']
            result['slug'] = chart_config.slug
        charts_data.append(result)

    return charts_data


def send_daily_digest():
    title = daily_digest_config.title
    today = daily_digest_config.timezone.localize(current_time_naive())
    subject = '{} - {}'.format(title, today.strftime('%x'))
    text_content = title

    context = {
        'charts': charts_data_for_config(chart_format='png')
    }

    html_template = get_template('daily_digest/email.html')
    html_content = html_template.render(context)
    # Inline all css
    html_content = transform(html_content)

    msg = EmailMultiAlternatives(subject, text_content, daily_digest_config.from_email, settings.ADMINS)
    msg.attach_alternative(html_content, "text/html")

    for chart_config in daily_digest_config.chart_configs:
        with open('/tmp/{}.png'.format(chart_config.slug), 'rb') as image_file:
            msg_image = MIMEImage(image_file.read())
            msg_image.add_header('Content-ID', '<{}>'.format(chart_config.slug))
            msg.attach(msg_image)

    msg.send(fail_silently=False)



# -*- coding: utf-8 -*-
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect
from django.shortcuts import render

from .config import daily_digest_config
from .utils import charts_data_for_config


@login_required
def preview_daily_digest(request):
    if not request.user.is_staff:
        return HttpResponseRedirect("/admin/login/?next=" + request.path)

    context = {
        'title': daily_digest_config.title,
        'charts': charts_data_for_config()
    }
    return render(request, 'daily_digest/email.html', context)



import random
from datetime import date, datetime, timedelta

import mock
import pytz
from django.contrib.auth.models import User
from django.core.management import call_command
from django.test import TestCase
from django.test.client import Client
from project.photos.models import PhotoUpload

from .utils import (
    EmailMultiAlternatives, send_daily_digest, series_data_for_model
)


class MockEmailMultiAlternatives(mock.Mock):
    subject = ''

    def __init__(self, subject, text_content, from_email, to):
        self.subject = subject
        self.text_content = text_content
        self.from_email = from_email
        self.to = to


class DailyDigestTestCase(TestCase):

    @mock.patch('daily_digest.utils.current_time_naive', return_value=datetime(2018, 1, 10, 8, 0))
    @mock.patch.object(EmailMultiAlternatives, 'send')
    @mock.patch.object(EmailMultiAlternatives, 'attach')
    @mock.patch.object(EmailMultiAlternatives, 'attach_alternative')
    @mock.patch.object(EmailMultiAlternatives, '__init__', return_value=None)
    def test_send_daily_digest(self, mock_init_email, mock_attach_html, mock_add_attachment, *_):
        send_daily_digest()

        subject, text_content, from_email, to_emails = mock_init_email.call_args_list[0][0]
        self.assertEqual(subject, 'Daily Digest - 01/10/18')
        self.assertEqual(text_content, 'Daily Digest')
        self.assertEqual(from_email, 'support@test.com')
        self.assertEqual(list(to_emails), [])
        self.assertEqual(
            mock_attach_html.call_args_list[0],
            mock.call('<!DOCTYPE html>\n<html>\n<head>\n\t<title></title>\n\t<meta name="viewport" content="width=device-width, initial-scale=1.0">\n\n\t</head>\n<body style="font-family:-apple-system, BlinkMacSystemFont, sans-serif">\n\t\n\n\n<h3 style="font-weight:500; margin-bottom:0">New Users</h3>\n\n\n\t<img class="chart" width="100%" src="cid:new-users" style="max-width:480px">\n\n<br>\n\n<h3 style="font-weight:500; margin-bottom:0">Photo Uploads</h3>\n\n\n\t<img class="chart" width="100%" src="cid:photo-uploads" style="max-width:480px">\n\n<br>\n\n\n\n</body>\n</html>\n', 'text/html')
        )
        self.assertEqual(mock_add_attachment.call_args_list[0][0][0].get('Content-ID'), '<new-users>')

    @mock.patch('daily_digest.management.commands.send_daily_digest.send_daily_digest')
    def test_send_daily_digest_command(self, mock_send_digest):
        call_command('send_daily_digest')

        self.assertEqual(mock_send_digest.called, True)

    def test_daily_digest_preview(self):
        user = User.objects.create(username='test', is_superuser=True, is_staff=True)
        user.set_password('test')
        user.save()

        client = Client()
        client.login(username='test', password='test')

        response = client.get('/admin/daily-digest-preview/')
        self.assertEqual(response.status_code, 200)

    def test_daily_digest_preview_requires_staff(self):
        user = User.objects.create(username='test')
        user.set_password('test')
        user.save()

        client = Client()
        client.login(username='test', password='test')

        response = client.get('/admin/daily-digest-preview/')
        self.assertEqual(response.status_code, 302)

    @mock.patch('daily_digest.utils.current_time_naive', return_value=datetime(2018, 1, 10, 16, 0))  # 8am LA
    def test_series_data_generated_for_current_period(self, *_):
        timezone = pytz.timezone('America/Los_Angeles')

        User.objects.filter(username__startswith='test').delete()
        PhotoUpload.objects.all().delete()
        los_angeles_timezone = pytz.timezone('America/Los_Angeles')
        today = los_angeles_timezone.localize(datetime(2018, 1, 10, 16, 0))

        for days in [-1, 0, 1, 2, 3, 4, 5, 6, 7]:
            for _ in range(0, days + 3):
                User.objects.create(username='test-{}'.format(random.randint(0, 100000000)), date_joined=today - timedelta(days=days))

        queryset = User.objects.all()

        data, total_count = series_data_for_model(
            queryset, 'date_joined', timezone
        )
        self.assertEqual(total_count, 42)
        self.maxDiff = None
        self.assertEqual(data, [
            [date(2018, 1, 4), 9], [date(2018, 1, 5), 8], [date(2018, 1, 6), 7],
            [date(2018, 1, 7), 6], [date(2018, 1, 8), 5], [date(2018, 1, 9), 4],
            [date(2018, 1, 10), 3]
        ])

    @mock.patch('daily_digest.utils.current_time_naive', return_value=datetime(2018, 1, 10, 16, 0))  # 8am LA
    def test_series_data_generated_for_previous_period(self, *_):
        timezone = pytz.timezone('America/Los_Angeles')

        User.objects.filter(username__startswith='test').delete()
        PhotoUpload.objects.all().delete()
        los_angeles_timezone = pytz.timezone('America/Los_Angeles')
        today = los_angeles_timezone.localize(datetime(2018, 1, 10, 16, 0))

        for days in [6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]:
            for _ in range(0, days):
                User.objects.create(username='test-{}'.format(random.randint(0, 100000000)), date_joined=today - timedelta(days=days))

        queryset = User.objects.all()

        data, total_count = series_data_for_model(
            queryset, 'date_joined', timezone, prev_period=True
        )
        self.assertEqual(total_count, 70)
        self.maxDiff = None
        self.assertEqual(data, [
            [date(2018, 1, 4), 13], [date(2018, 1, 5), 12], [date(2018, 1, 6), 11],
            [date(2018, 1, 7), 10], [date(2018, 1, 8), 9], [date(2018, 1, 9), 8],
            [date(2018, 1, 10), 7]
        ])



from django.contrib.auth.models import User
from django.db import models


class PhotoUpload(models.Model):
    user = models.ForeignKey(User, related_name='photo_uploads', on_delete=models.CASCADE)

    created = models.DateTimeField(auto_now_add=True)



import os
import sys

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))

UNIT_TESTING = 'test' in sys.argv

DEBUG = True
ALLOWED_HOSTS = []
SECRET_KEY = 'test'

LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True
USE_TZ = True

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite3' if UNIT_TESTING else 'db.sqlite3'
    }
}

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]
MIDDLEWARE_CLASSES = MIDDLEWARE  # Django 1.8 support

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.sessions',
    'django.contrib.contenttypes',
    'django.contrib.staticfiles',

    # Used for examples
    'project.photos',

    'daily_digest'
]

ROOT_URLCONF = 'project.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ]
        }
    },
]

LOGIN_URL = '/admin/'

STATIC_URL = '/static/'

DAILY_DIGEST_CONFIG = {
    'title': 'Daily Digest',
    'from_email': 'support@test.com',
    'timezone': 'America/Los_Angeles',
    'exclude_today': False,
    'charts': [
        {
            'title': 'New Users',
            'app_label': 'auth',
            'model': 'user',
            'date_field': 'date_joined',
            'filter_kwargs': {
                'is_active': True
            }
        },
        {
            'title': 'Photo Uploads',
            'app_label': 'photos',
            'model': 'photoupload',
            'date_field': 'created'
        },
    ]
}



import pytz
from collections import namedtuple
from django.conf import settings
from django.utils.text import slugify


class DailyDigestConfigurationException():
    pass


class DailyDigestRequiredFieldException(DailyDigestConfigurationException):
    pass


_config = getattr(settings, 'DAILY_DIGEST_CONFIG', {})


DailyDigestChart = namedtuple('DailyDigestChart', ['title', 'slug', 'app_label', 'model', 'date_field', 'filter_kwargs', 'distinct_by'])
_chart_configs = []
for chart in _config.get('charts', []):
    for key in ['title', 'app_label', 'model', 'date_field']:
        if not chart.get(key):
            raise DailyDigestRequiredFieldException('Missing required field {} for chart config.'.format(key))
    chart_config = DailyDigestChart(
        title=chart['title'],
        slug=slugify(chart['title']),
        app_label=chart['app_label'],
        model=chart['model'],
        date_field=chart['date_field'],
        filter_kwargs=chart.get('filter_kwargs', {}),
        distinct_by=chart.get('distinct_by')
    )
    _chart_configs.append(chart_config)


DailyDigestConfig = namedtuple('DailyDigestConfig', ['title', 'from_email', 'timezone', 'exclude_today', 'chart_configs'])
daily_digest_config = DailyDigestConfig(
    title=_config.get('title', 'Daily Digest'),
    from_email=_config.get('from_email', settings.DEFAULT_FROM_EMAIL),
    timezone=pytz.timezone(_config['timezone']) if _config.get('timezone') else pytz.UTC,
    exclude_today=_config.get('exclude_today', False),
    chart_configs=_chart_configs
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
from setuptools import find_packages, setup, Command
from shutil import rmtree


NAME = 'daily_digest'
DESCRIPTION = 'Simple daily summary email with charts.'
URL = 'https://github.com/nickromano/django-daily-digest'
EMAIL = 'nick.r.romano@gmail.com'
AUTHOR = 'Nick Romano'

here = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(here, NAME, '__init__.py')) as f:
    exec(f.read(), about)


class PublishCommand(Command):
    """Support setup.py publish."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Print things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.sep.join(('.', 'dist')))
        except FileNotFoundError:  # noqa
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload dist/*')

        sys.exit()


setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=DESCRIPTION,
    author=AUTHOR,
    author_email=EMAIL,
    url=URL,
    packages=find_packages(exclude=('tests',)),
    license='MIT',
    package_data={'daily_digest': ['templates/daily_digest/*.html']},
    install_requires=[
        'Django>=1.8',
        'CairoSVG>=2.0.3',
        'premailer',
    ],
    # Dependency links are no longer supported by pip so the leather code has been copied to this repo
    # until this PR is merged: https://github.com/wireservice/leather/pull/89.
    # dependency_links=[
    #     'git+ssh://git@github.com/nickromano/leather.git#egg=leather-0.3.3',
    # ],
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6'
    ],
    tests_require=[
        'mock'
    ],
    test_suite='testrunner.runtests',
    cmdclass={
        'publish': PublishCommand
    },
)
